# Copyright (C) 2022-2023 |Meso|Star> (contact@meso-star.com)
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

cmake_minimum_required(VERSION 3.1)
include(SelectLibraryConfigurations)

# Try to find the gmsh devel. Once done this will define:
#   - GMSH_FOUND: system has gmsh
#   - GMSH_INCLUDE_DIR: the include directory
#   - GMSH: Link this to use gmsh

# Look for library header
find_path(GMSH_INCLUDE_DIR gmsh/gmshc.h)

# Look for Release, Debug, RelWithDebInfo and MinSizeRel libraries
unset(GMSH_LIBRARY CACHE)
unset(GMSH_LIBRARY_RELEASE CACHE)
unset(GMSH_LIBRARY_DEBUG CACHE)
unset(GMSH_LIBRARY_RELWITHDEBINFO CACHE)
unset(GMSH_LIBRARY_MINSIZEREL CACHE)

# Find per configuration type libraries
find_library(GMSH_LIBRARY_RELEASE
  gmsh
  PATH_SUFFIXES bin Bin BIN
  DOC "Path to the library gmsh used during release builds.")
find_library(GMSH_LIBRARY_DEBUG
  gmsh-dbg
  PATH_SUFFIXES bin Bin BIN
  DOC "Path to the library gmsh used during debug builds.")
find_library(GMSH_LIBRARY_RELWITHDEBINFO
  gmsh-rdbg
  PATH_SUFFIXES bin Bin BIN
  DOC "Path to the library gmsh used during release with debug info builds.")
find_library(GMSH_LIBRARY_MINSIZEREL
  gmsh-mszr
  PATH_SUFFIXES bin Bin BIN
  DOC "Path to the library gmsh used during minsize builds.")

# Define the generic gmsh library
if(GMSH_LIBRARY_RELEASE)
  set(GMSH_LIBRARY ${GMSH_LIBRARY_RELEASE})
elseif(GMSH_LIBRARY_RELWITHDEBINFO)
  set(GMSH_LIBRARY ${GMSH_LIBRARY_RELWITHDEBINFO})
elseif(GMSH_LIBRARY_MINSIZEREL)
  set(GMSH_LIBRARY ${GMSH_LIBRARY_MINSIZEREL})
elseif(GMSH_LIBRARY_DEBUG)
  set(GMSH_LIBRARY ${GMSH_LIBRARY_DEBUG})
endif()

# Define the per configuration library fallback when not found
set(_configs RELEASE DEBUG RELWITHDEBINFO MINSIZEREL)
foreach(_cfg ${_configs})
  if(NOT GMSH_LIBRARY_${_cfg})
    get_property(_doc CACHE GMSH_LIBRARY_${_cfg} PROPERTY HELPSTRING)
    set(GMSH_LIBRARY_${_cfg}
      ${GMSH_LIBRARY} CACHE PATH ${_doc} FORCE)
  endif()
endforeach()

# Create the imported library target
add_library(gmsh SHARED IMPORTED)

# Setup the properties of the imported target
if(CMAKE_HOST_WIN32)
  set(_import_prop IMPORTED_IMPLIB)
else()
  set(_import_prop IMPORTED_LOCATION)
endif()
set_target_properties(gmsh PROPERTIES
  ${_import_prop} ${GMSH_LIBRARY}
  ${_import_prop}_RELEASE ${GMSH_LIBRARY_RELEASE}
  ${_import_prop}_DEBUG ${GMSH_LIBRARY_DEBUG}
  ${_import_prop}_RELWITHDEBINFO ${GMSH_LIBRARY_RELWITHDEBINFO}
  ${_import_prop}_MINSIZEREL ${GMSH_LIBRARY_MINSIZEREL}
  INTERFACE_INCLUDE_DIRECTORIES ${GMSH_INCLUDE_DIR})

# Check the package
include(FindPackageHandleStandardArgs)
FIND_PACKAGE_HANDLE_STANDARD_ARGS(gmsh DEFAULT_MSG
  GMSH_INCLUDE_DIR
  GMSH_LIBRARY)
